package ru.ldeloff.servermonitorbot.service.ssh;

import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.model.Command;
import ru.ldeloff.servermonitorbot.model.SshServer;
import ru.ldeloff.servermonitorbot.repository.SshRepository;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class SshServiceImpl implements SshService {
    final SshRepository sshRepository;
    @Override
    public SendMessage getStatusSessions(SendMessage message) {
        message.setText(sshRepository.getStatusSessions());
        return message;
    }
    @Override
    public List<SshServer> getSshServers() {
        return sshRepository.getSshServers();
    }

    @Override
    public Command execute(Command command) {
        List<Command> commands = new ArrayList<>();
        commands.add(command);
        return execute(commands).get(0);
    }

    @Override
    public List<Command> execute(List<Command> commands) {
        Session session = commands.get(0).getSshServer().getSession(); // подразумевается, что в листе с командами сервер один и тот же
        ChannelExec channel = null;
        try {
            channel = (ChannelExec) session.openChannel("exec");
            ChannelExec finalChannel = channel;

            commands.forEach(command -> {
                try {
                    finalChannel.setCommand(command.getCommand());
                    ByteArrayOutputStream responseStream = new ByteArrayOutputStream();
                    finalChannel.setOutputStream(responseStream);
                    finalChannel.connect();
                    while (finalChannel.isConnected()) {
                        Thread.sleep(command.getTimeout());
                    }
                    String responseString = new String(responseStream.toByteArray());
                    command.setResponse(responseString);
                } catch (JSchException | InterruptedException e) {
                    log.warn(e.getMessage());
                    throw new RuntimeException(e);
                }
            });
        } catch (JSchException e) {
            log.warn(e.getMessage());
            throw new RuntimeException(e);
        } finally {
            if (channel != null) {
                channel.disconnect();
            }
        }
        return commands;
    }
}
